@@ -1,22 +1,23 @@
-/* GMP-ECM -- Integer factorization with ECM and Pollard 'P-1' methods.
+/* GMP-ECM -- Integer factorization with ECM, P-1 and P+1 methods.
 
-  Copyright 2001, 2002, 2003, 2004, 2005, 2011 Jim Fougeron, Laurent Fousse, Alexander Kruppa, Paul Zimmermann.
+Copyright 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011,
+2012 Jim Fougeron, Laurent Fousse, Alexander Kruppa, Paul Zimmermann, Cyril
+Bouvier.
 
-  This program is free software; you can redistribute it and/or modify it
-  under the terms of the GNU General Public License as published by the
-  Free Software Foundation; either version 2 of the License, or (at your
-  option) any later version.
+This program is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 3 of the License, or (at your
+option) any later version.
 
-  This program is distributed in the hope that it will be useful, but WITHOUT
-  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
-  more details.
+This program is distributed in the hope that it will be useful, but
+WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
+more details.
 
-  You should have received a copy of the GNU General Public License along
-  with this program; see the file COPYING.  If not, write to the Free
-  Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
-  02111-1307, USA.
-*/
+You should have received a copy of the GNU General Public License
+along with this program; see the file COPYING.  If not, see
+http://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
+51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA. */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -64,91 +65,6 @@ static  char *prpcmd = NULL;
 static int exit_asap_value = 0;
 static int exit_asap_signalnr = 0; /* Remembers which signal we received */
 
-/* Tries to read a number from a line from fd and stores it in r.
-   Keeps reading lines until a number is found. Lines beginning with "#"
-     are skipped.
-   Returns 1 if a number was successfully read, 0 if no number can be read
-     (i.e. at EOF)
-   Function is now simpler.  Much of the logic (other than skipping # lines
-     is now contained within eval() function.
-*/
-
-int
-read_number (mpcandi_t *n, FILE *fd, int primetest)
-{
-  int c;
-
-new_line:
-  c = fgetc (fd);
-
-  /* Skip comment lines beginning with '#' */
-  if (c == '#')
-    {
-      do
-        c = fgetc (fd);
-      while (c != EOF && !IS_NEWLINE(c));
-      if (IS_NEWLINE(c))
-        goto new_line;
-    }
-
-  if (c == EOF)
-    return 0;
-
-  ungetc (c, fd);
-  if (!eval (n, fd, primetest))
-    goto new_line;
-
-#if 0
-  /*  Code to test out eval_str function, which "appears" to work correctly. */
-  {
-    /* warning!! Line is pretty small, but since this is just testing code, we
-       can easily control the input for this test.  This code should NEVER be
-       compiled into released build, its only for testing of eval_str() */
-    char Line[500], *cp;
-    fgets (Line, sizeof(Line), fd);
-
-    if (!eval_str (n, Line, primetest, &cp))
-      goto new_line;
-    fprintf (stderr, "\nLine is at %X cp is at %X\n", Line, cp);
-  }
-#endif
-
-#if defined (DEBUG_EVALUATOR)
-  if (n->cpExpr)
-    fprintf (stderr, "%s\n", n->cpExpr);
-  mpz_out_str (stderr, 10, n->n);
-  fprintf (stderr, "\n");
-#endif
-
-  return 1;
-}
-
-int
-probab_prime_p (mpz_t N, int reps)
-{
-#ifdef WANT_SHELLCMD
-  if (prpcmd != NULL)
-    {
-      FILE *fc;
-      int r;
-      fc = popen (prpcmd, "w");
-      if (fc != NULL)
-        {
-          gmp_fprintf (fc, "%Zd\n", N);
-          r = pclose (fc);
-          if (r == 0) /* Exit status of 0 means success = is a PRP */
-            return 1;
-          else
-            return 0;
-        } else {
-          fprintf (stderr, "Error executing the PRP command\n");
-          exit (EXIT_FAILURE);
-        }
-    } else
-#endif
-      return mpz_probab_prime_p (N, reps);
-}
-
 void
 signal_handler (int sig)
 {
@@ -207,7 +123,7 @@ usage (void)
     printf ("  -resume file resume residues from file, reads from stdin if file is \"-\"\n");
     printf ("  -chkpnt file save periodic checkpoints during stage 1 to file\n");
     printf ("  -primetest   perform a primality test on input\n");
-    printf ("  -treefile f  store product tree of F in files f.0 f.1 ... \n");
+    printf ("  -treefile f  [ECM only] store stage 2 data in files f.0, ... \n");
     printf ("  -maxmem n    use at most n MB of memory in stage 2\n");
     printf ("  -stage1time n add n seconds to ECM stage 1 time (for expected time est.)\n");
 #ifdef WANT_SHELLCMD
@@ -226,7 +142,6 @@ usage (void)
     printf ("  -one         Stop processing a candidate if a factor is found (looping mode)\n");
     printf ("  -n           run ecm in \"nice\" mode (below normal priority)\n");
     printf ("  -nn          run ecm in \"very nice\" mode (idle priority)\n");
-    printf ("  -t n         Trial divide candidates before P-1, P+1 or ECM up to n\n");
     printf ("  -ve n        Verbosely show short (< n character) expressions on each loop\n");
     printf ("  -cofdec      Force cofactor output in decimal (even if expressions are used)\n");
     printf ("  -B2scale f   Multiplies the default B2 value by f \n");
@@ -234,8 +149,10 @@ usage (void)
     printf ("               or can use N as a placeholder for the number being factored.\n");
     printf ("  -printconfig Print compile-time configuration and exit.\n");
 
-    printf ("  -batch       (experimental) use Montgomery parametrization and batch\n" 
-					  "               computation.\n");
+    printf ("  -batch[=1|2] (experimental) use Montgomery parametrization and batch\n" 
+					  "               computation. Option -batch is equivalent to -batch=1\n");
+    printf ("  -bsaves file In the batch mode, save s in file.\n");
+    printf ("  -bloads file In the batch mode, load s from file.\n");
 
     printf ("  -h, --help   Prints this help and exit.\n");
 }
@@ -266,7 +183,7 @@ print_config ()
 #endif
 
 #ifdef HAVE_SSE2
-  printf ("HAVE_SSE2 = %d\n$", HAVE_SSE2);
+  printf ("HAVE_SSE2 = %d\n", HAVE_SSE2);
 #else
   printf ("HAVE_SSE2 undefined\n");
 #endif
@@ -297,16 +214,6 @@ print_config ()
 
 #ifdef USE_ASM_REDC
   printf ("USE_ASM_REDC = %d\n", USE_ASM_REDC);
-#ifdef TUNE_MULREDC_THRESH
-  printf ("TUNE_MULREDC_THRESH = %d\n", TUNE_MULREDC_THRESH);
-#else
-  printf ("TUNE_MULREDC_THRESH undefined\n");
-#endif
-#ifdef TUNE_SQRREDC_THRESH
-  printf ("TUNE_SQRREDC_THRESH = %d\n", TUNE_SQRREDC_THRESH);
-#else
-  printf ("TUNE_SQRREDC_THRESH undefined\n");
-#endif
 #ifdef WINDOWS64_ABI
   printf ("WINDOWS64_ABI = %d\n", WINDOWS64_ABI);
 #else
@@ -403,7 +310,7 @@ int
 main (int argc, char *argv[])
 {
   char **argv0 = argv;
-  mpz_t x, sigma, A, f, orig_x0, B2, B2min, startingB2min;
+  mpz_t seed, x, sigma, A, f, orig_x0, B2, B2min, startingB2min;
   mpcandi_t n;
   mpgocandi_t go;
   mpq_t rat_x0;
@@ -448,12 +355,13 @@ main (int argc, char *argv[])
   int deep=1, trial_factor_found;
   unsigned int displayexpr = 0;
   unsigned int decimal_cofactor = 0;
-  double maxtrialdiv = 0.0;
   double B2scale = 1.0;
   double maxmem = 0.;
   double stage1time = 0.;
   ecm_params params;
   int batch = 0; /* By default we don't use batch mode */
+  char *savefile_s = NULL;
+  char *loadfile_s = NULL;
 #ifdef WANT_SHELLCMD
   char *faccmd = NULL;
   char *idlecmd = NULL;
@@ -483,6 +391,7 @@ main (int argc, char *argv[])
   mpgocandi_t_init (&go);
 
   /* Init variables we might need to store options */
+  MPZ_INIT (seed);
   MPZ_INIT (sigma);
   MPZ_INIT (A);
   MPZ_INIT (B2);
@@ -583,12 +492,31 @@ main (int argc, char *argv[])
 	  argv++;
 	  argc--;
         }
-      else if (strcmp (argv[1], "-batch") == 0)
+      else if (strcmp (argv[1], "-batch") == 0 || 
+                                            strcmp (argv[1], "-batch=1") == 0)
         {
 	  batch = 1;
 	  argv++;
 	  argc--;
         }
+      else if (strcmp (argv[1], "-batch=2") == 0)
+        {
+	  batch = 2;
+	  argv++;
+	  argc--;
+        }
+      else if ((argc > 2) && (strcmp (argv[1], "-bsaves") == 0))
+        {
+          savefile_s = argv[2];
+          argv += 2;
+          argc -= 2;
+        }
+      else if ((argc > 2) && (strcmp (argv[1], "-bloads") == 0))
+        {
+          loadfile_s = argv[2];
+          argv += 2;
+          argc -= 2;
+        }
       else if (strcmp (argv[1], "-h") == 0 || strcmp (argv[1], "--help") == 0)
         {
           usage ();
@@ -763,17 +691,6 @@ main (int argc, char *argv[])
 	  argv += 2;
 	  argc -= 2;
 	}
-      else if ((argc > 2) && (strcmp (argv[1], "-t") == 0))
-	{
-	  maxtrialdiv = strtod (argv[2], NULL);
-	  if (maxtrialdiv <= 0.0)
-	    {
-	      fprintf (stderr, "Error, the -t option requires a positive argument\n");
-	      exit (EXIT_FAILURE);
-  	    }
-	  argv += 2;
-	  argc -= 2;
-	}
       else if ((argc > 2) && (strcmp (argv[1], "-ve") == 0))
         {
 	  displayexpr = atoi (argv[2]);
@@ -1064,6 +981,13 @@ main (int argc, char *argv[])
   params->maxmem = maxmem;
   params->stage1time = stage1time;
 
+  /* -treefile is valid for ECM only */
+  if (TreeFilename != NULL && method != ECM_ECM)
+    {
+      fprintf (stderr, "Error: the -treefile option is for ECM only\n");
+      exit (EXIT_FAILURE);
+    }
+
   /* Open resume file for reading, if resuming is requested */
   if (resumefilename != NULL)
     {
@@ -1119,7 +1043,15 @@ main (int argc, char *argv[])
 
   /* We may need random numbers for sigma/starting point */
   gmp_randinit_default (randstate);
-  gmp_randseed_ui (randstate, get_random_ui ());
+  mpz_set_ui (seed, get_random_ul ());
+  if (mpz_sizeinbase (seed, 2) <= 32)
+    {
+      mpz_mul_2exp (seed, seed, 32);
+      mpz_add_ui (seed, seed, get_random_ul ());
+    }
+  if (verbose >= 3)
+    gmp_printf ("Random seed: %Zd\n", seed);
+  gmp_randseed (randstate, seed);
 
 
   /* Install signal handlers */
@@ -1418,43 +1350,6 @@ BreadthFirstDoAgain:;
 	  FREE (s, n.ndigits + 1);
 	}
 
-      if ((!breadthfirst && cnt == count) || 
-          (breadthfirst && 1 == breadthfirst_cnt))
-	{
-	  int SomeFactor;
-	  /*  Note, if a factors are found, then n will be adjusted "down" */
-	  SomeFactor = trial_factor (&n, maxtrialdiv, deep);
-	  if (SomeFactor)
-	    {
-	      /* should we increase factors found for trivials ??? */
-	      trial_factor_found = 1;
-	      factsfound += SomeFactor;
-	      if (n.isPrp)
-	        {
-		  printf ("Probable prime cofactor ");
-		  if (n.cpExpr && !decimal_cofactor)
-		    printf ("%s", n.cpExpr);
-		  else
-		    mpz_out_str (stdout, 10, n.n);
-		  printf (" has %u digits\n", n.ndigits);
-		  /* Nothing left to do with this number, so simply continue. */
-		  cnt = 0; /* no more curve to perform */
-		  fflush (stdout);
-		  continue;
-		}
-	      fflush (stdout);
-	      if (!deep)
-		{
-		  /* Note, if we are not in deep mode, then there is no need 
-		     to continue if a factor was found */
-  		  factor_is_prime = 1;
-  		  mpz_set_ui (f,1);
-		  goto OutputFactorStuff;
-		}
-
-	    }
-        }
-
       factor_is_prime = 0;
 
       cnt --; /* one more curve performed */
@@ -1464,29 +1359,15 @@ BreadthFirstDoAgain:;
          If A was given one should check that d fits in one word and that x0=2.
          If A was not given one chooses it at random (and if x0 exists
          it must be 2). */
-      if (batch == 1)
+      if (batch != 0)
         {
-          static int random = 0; /* non-zero if user asked for random curves */
-
           if (method != ECM_ECM)
             {
               fprintf (stderr, "Error, the -batch option is only valid for ECM\n");
               exit (EXIT_FAILURE);
             }
           mpz_set_ui (sigma, 0); 
-          if (random || (mpz_sgn (A) == 0)) /* A was not given by the user */
-            {
-              random = 1;
-              /* We need that d = (A+2)/4 is smaller than 2^GMP_NUMB_BITS */
-              mpz_urandomb (A, randstate, 32);  /* generates d */
-              if (GMP_NUMB_BITS >= 64)
-                mpz_mul (A, A, A);              /* ensures d is a square,
-                                                   which increases the success
-                                                   probability */
-              mpz_mul_2exp (A, A, 2);           /* 4d */
-              mpz_sub_ui (A, A, 2);             /* 4d-2 */
-            }
-          
+
           if (mpz_sgn (orig_x0) == 0)
             mpz_set_ui (orig_x0, 2);
           else if (mpz_cmp_ui (orig_x0, 2) != 0)
@@ -1498,26 +1379,44 @@ BreadthFirstDoAgain:;
           
           mpz_set (x, orig_x0);
         }
-      /* set parameters that may change from one curve to another */
       params->batch = batch;
-      if (params->batch == 1 && params->batch_B1 != B1)
+      if (params->batch != 0 && params->batch_B1 != B1)
         {
           int st;
           params->batch_B1 = B1;
 
           if (verbose > OUTPUT_NORMAL)
-            printf ("Batch mode: \n");
+            printf ("Batch mode %d: ", batch);
           st = cputime ();
           /* construct the batch exponent */
-          compute_s (params->batch_s, params->batch_B1);
-          if (verbose > OUTPUT_NORMAL)
-            printf ("  computing prime product of %lu bits took %ldms\n",
+          if (loadfile_s != NULL)
+            {
+            /* For now, there is no check that it correspond to the actual B1*/
+              read_s_from_file (params->batch_s, loadfile_s);
+              if (verbose > OUTPUT_NORMAL)
+                printf ("reading prime product of %zu bits took %ldms\n",
                     mpz_sizeinbase (params->batch_s, 2), cputime () - st);
+            }
+          else
+            {
+              compute_s (params->batch_s, params->batch_B1);
+              if (verbose > OUTPUT_NORMAL)
+                printf ("computing prime product of %zu bits took %ldms\n",
+                    mpz_sizeinbase (params->batch_s, 2), cputime () - st);
+              if (savefile_s != NULL)
+                {
+                  int ret = write_s_in_file (savefile_s, params->batch_s);
+                  if (verbose > OUTPUT_NORMAL && ret > 0)
+                    printf ("Save s (%u bytes) in %s.\n", ret, savefile_s);
+                }
+            }
         }
+
+      /* set parameters that may change from one curve to another */
       params->method = method; /* may change with resume */
       mpz_set (params->x, x); /* may change with resume */
       /* if sigma is zero, then we use the A value instead */
-      params->sigma_is_A = ((mpz_sgn (sigma) == 0 || batch==1)? 1 : 0);
+      params->sigma_is_A = ((mpz_sgn (sigma) == 0 || batch != 0) ? 1 : 0);
       mpz_set (params->sigma, (params->sigma_is_A) ? A : sigma);
       mpz_set (params->go, go.Candi.n); /* may change if contains N */
       mpz_set (params->B2min, B2min); /* may change with -c */
@@ -1816,6 +1715,7 @@ OutputFactorStuff:;
   mpz_clear (sigma);
   mpz_clear (A);
   mpq_clear (rat_x0);
+  mpz_clear (seed);
   mpgocandi_t_free (&go);
 
   ecm_clear (params);
